import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;

public class Player extends Thread
{	private Socket connection;
	private DataInputStream input;
	private DataOutputStream output;
	private Server server;
	private int player;
	private Game game;
	private boolean hasAGame;
	protected boolean threadSuspended=true;
	private String name;
	private boolean ended;
	public Player(Socket s,Server t)
	{	connection=s;ended=false;
		try
		{	input=new DataInputStream(connection.getInputStream());
			output=new DataOutputStream(connection.getOutputStream());
		}catch(IOException e){
			e.printStackTrace();
			System.exit(1);
		}
		server=t;
		hasAGame=false;
	}
	public boolean equals(Player p)
	{	return name.equals(p.getPlayerName());
	}
	public void otherPlayerMoved(int i,int j,int x,int y,String dropType)
	{	sendString("Opponent moved");
	   	sendInt(i);
	   	sendInt(j);
	   	sendInt(x);
	   	sendInt(y);
	   	sendString(dropType);
	}
	public void run()
	{	name=getString();
		sendString(server.seekStrings());
		sendString(server.playerStrings());
		server.playersUpdated();
		while(!ended)processMessage(getString());
	}
	public void processMessage(String s)
	{	String already,notYet;
		already="You are already playing a game.";
		notYet="You are not playing a game.";
		if(s.equals("Move"))
		{	if(hasAGame)
			{	int x,y,i,j;
				i=getInt();
				j=getInt();
				x=getInt();
				y=getInt();
				String dt=getString();
				if(game.validMove(player,i,j,x,y,dt))
					sendString("Valid");
				else sendString("Invalid");
			}else sendString(notYet);	
		}else if(s.equals("Resign"))
		{	if(hasAGame)
			{	game.resign(player);
			}else sendString(notYet);	
		}else if(s.equals("Take Back Request"))
		{	if(hasAGame)
			{	game.takeBackRequest(this);
			}else sendString(notYet);	
		}else if(s.equals("Draw Offer"))
		{	if(hasAGame)
			{	game.drawOffer(this);
			}else sendString(notYet);	
		}else if(s.equals("Takeback Accepted"))
		{	if(hasAGame)
			{	game.takeBackAccepted(this);
			}else sendString(notYet);	
		}else if(s.equals("Draw Accepted"))
		{	if(hasAGame)
			{	game.drawAccepted();
			}else sendString(notYet);	
		}
		else if(s.equals("Seek"))
		{	if(!hasAGame)
			{	String g,b;
				g=getString();
				b=getString();
				boolean r=getBoolean();
				if(!server.seekExists(name,g,b,r))
					server.addASeek(this,g,b,r);
				else sendString(
						"You already have a seek matching those paramaters.");
			}else sendString(already);	
		}else if(s.equals("Seek Confirmed"))
		{	if(!hasAGame)
			{	int number=getInt();
				String p=getString();
				if(server.seekExists(number)&&
						server.playerExists(p)&&
						!name.equals(p))
				{	Seek seek=server.getSeek(number);
					Player other=server.getPlayer(p);
					server.addAGame(seek,other);
				}else sendString("Seek confirmation error.");
			}else sendString(already);
		}else if(s.equals("Accept Seek"))
		{	if(!hasAGame)
			{	int number=getInt();
				if(server.seekExists(number))
				{	server.getSeek(number).
						getPlayer().playerAccepted(name,number);
				}else sendString("No such seek exists.");
			}else sendString(already);
		}else if(s.equals("Seek Declined"))
		{	int n=getInt();
			String p=getString();
			Seek k=server.getSeek(n);
			server.getPlayer(p).
				seekDeclined(name,k.getGameType(),
					k.getBoardType(),k.getRandomized());
		}
		else if(s.equals("Withdraw Seek"))
		{	server.removeSeek(getInt());
		}else if(s.equals("Show Seeks"))
			sendString(server.seekStrings());
		else if(s.equals("Chat"))
			server.chat(name+": "+getString());
		else if(s.equals("Game Chat"))
			game.chat(name+": "+getString());
	}
	public String getPlayerName()
	{	return name;
	}
	public void resign(int j)
	{	sendString("Resign");
		sendInt(j);
	}
	public void setGame(Game g,int p,String pl)
	{	game=g;hasAGame=true;player=p;
		sendString("Game Started");
		sendInt(player);
		Board b=g.board();
		sendString(b.boardName);
		sendBoolean(b.randomized);
		sendString(b.gameType);
		if(b.randomized)
		{	String[] pieces=g.getPieces();
			int l=pieces.length;sendInt(l);
			for(int i=0;i<l;i++)sendString(pieces[i]);
		}
		sendString(pl);
	}
	public void chat(String s)
	{	sendString("Chat");
		sendString(s);
	}
	public void endGame()
	{	hasAGame=false;
		sendString("Game Over");
		sendString(game.board().message());
		server.removeGame(game);
	}
	public void abortGame()
	{	if(!ended)
		{	hasAGame=false;
			sendString("Game Aborted");
			server.removeGame(game);
		}
	}
	public void takeBackMove(int ammt)
	{	sendString("Move Taken Back");
		sendInt(ammt);
	}
	public void playersUpdated(String s)
	{	sendString("Players Updated");
		sendString(s);
	}
	public void seeksUpdated(String s)
	{	sendString("Seeks Updated");
		sendString(s);
	}
	public boolean getHasAGame()
	{	return hasAGame;
	}
	public Game getGame()
	{	return game;
	}
	public synchronized void playerAccepted(String n,int i)
	{	sendString("Player Accepted");
		sendInt(i);
		sendString(n);
	}
	public synchronized void youAddedSeek(
			int number,String gameType,String map,boolean rand)
	{	sendString("You Added A Seek");
		sendInt(number);
		sendString(gameType);
		sendString(map);
		sendBoolean(rand);
	}
	public synchronized void seekDeclined(String p,String g,String m,boolean r)
	{	String sendM=m;
		if(r)m+="(r)";
		sendString("Seek Declined");
		sendString(p);
		sendString(g);
		sendString(sendM);
	}
	public void sendString(String s)
	{	try 
		{	server.debugOutput(name,s);
			output.writeUTF(s);
		}catch(IOException e)
		{	quit();
		}
	}
	public void sendInt(int i)
	{	try 
		{	server.debugOutput(name,i+"");
			output.writeInt(i);
		}catch(IOException e)
		{	quit();
		}
	}
	public void sendBoolean(boolean b)
	{	try 
		{	server.debugOutput(name,b+"");
			output.writeBoolean(b);
		}catch(IOException e)
		{	quit();
		}
	}
	public String getString()
	{	try 
		{	String i=input.readUTF();
			server.debugInput(name,i);
			return i;
		}catch(IOException e)
		{	quit();
			return "Error";
		}
	}
	public int getInt()
	{	try 
		{	int i=input.readInt();
			server.debugInput(name,i+"");
			return i;
		}catch(IOException e)
		{	quit();
			return -1;
		}
	}
	public boolean getBoolean()
	{	try 
		{	boolean b=input.readBoolean();
			server.debugInput(name,b+"");
			return b;
		}catch(IOException e)
		{	quit();
			return false;
		}
	}
	public void quit()
	{	server.removePlayer(this);
	}
	public void end()
	{	ended=true;
	}
}                  