import java.io.IOException;
import java.net.ServerSocket;
import java.util.ArrayList;

public class Server
{	public static void main(String args[])
	{	Server serv=new Server();
		serv.execute();
	}
	private ArrayList players; //<Player>
	private ArrayList games; //<Game>
	private ArrayList seeks; //<Seek>
	private ServerSocket server;
	private DebugWindow debug;
	static final long serialVersionUID=(long)0.0;
	public Server()
	{  	players=new ArrayList(); //<Player>
		games=new ArrayList(); //<Game>
		seeks=new ArrayList(); //<Seek>
		debug=new DebugWindow("Server Debug");
      	try
      	{	server=new ServerSocket(8008,1000);
      	}catch(IOException e){
      		e.printStackTrace();
      		System.exit(1);
      	}
      	display("Server running...");
	}
	public void execute()
	{	while(true)
		{	try
			{	Player toAdd=new Player(server.accept(),this);
				players.add(toAdd);
				toAdd.start();
			}catch(IOException e)
			{	e.printStackTrace();
				System.exit(1);
			}
		}
	}
	public void display(String s)
	{	System.out.println(s);
	}
	public void display(int s)
	{	System.out.println(s);
	}
	public synchronized void addAGame(Seek s,Player p2)
	{	Player p1=s.getPlayer();
		Player[] p={p1,p2};
		int gameNum=1;
		while(gameExists(gameNum))gameNum++;
		Board b=s.generateBoard();
		Game g=new Game(this,b,p,games.size(),s.getPieces());
		games.add(g);
		String pl=g.playerStrings();
		p1.setGame(g,1,pl);
		p2.setGame(g,2,pl);
		removeSeek(s);
		seeksUpdated();
		removeAllSeeks(p1);
		removeAllSeeks(p2);
	}
	public synchronized void removeAllSeeks(Player p)
	{	for(int i=0;i<seeks.size();i++)
		{	Seek s=(Seek)seeks.get(i);
			if(p.equals(s.getPlayer()))
				removeSeek(s);
		}
	}
	public synchronized void removePlayer(Player p)
	{	removeAllSeeks(p);
		if(p.getHasAGame())p.getGame().abortGame();
		players.remove(p);
		p.end();
		playersUpdated();
	}
	public synchronized void removeGame(Game g)
	{	games.remove(g);
	}
	public synchronized void removeSeek(int n)
	{	if(seekExists(n))
			removeSeek(getSeek(n));
	}
	public synchronized void removeSeek(Seek s)
	{	seeks.remove(s);
		seeksUpdated();
	}
	public synchronized void addASeek(Player p,String g,String b,boolean r)
	{	int seekNum=1;
		while(seekExists(seekNum))seekNum++;
		Seek s=new Seek(p,g,b,r,seekNum);
		seeks.add(s);
		seeksUpdated();
		p.youAddedSeek(seekNum,g,b,r);
	}
	public boolean gameExists(int gameNum)
	{	for(int i=0;i<games.size();i++)
			if(((Game)games.get(i)).getNumber()==gameNum)return true;
		return false;
	}
	public boolean seekExists(int seekNum)
	{	for(int i=0;i<seeks.size();i++)
			if(((Seek)seeks.get(i)).getNumber()==seekNum)return true;
		return false;
	}
	public boolean seekExists(String name,String gameType,
			String mapType,boolean r)
	{	for(int i=0;i<seeks.size();i++)
		{	Seek s=(Seek)seeks.get(i);
			if(s.playerName().equals(name)&&
					s.getGameType().equals(gameType)&&
					s.getBoardType().equals(mapType)&&
					s.getRandomized()==r)
				return true;
		}
		return false;
	}
	public boolean playerExists(String name)
	{	for(int i=0;i<players.size();i++)
			if(((Player)players.get(i)).getPlayerName().equals(name))
				return true;
		return false;
	}
	public Seek getSeek(int n)
	{	for(int i=0;i<seeks.size();i++)
			if(((Seek)seeks.get(i)).getNumber()==n)
				return (Seek)seeks.get(i);
		return (Seek)seeks.get(0);
	}
	public Game getGame(int n)
	{	for(int i=0;i<games.size();i++)
			if(((Game)games.get(i)).getNumber()==n)
				return (Game)games.get(i);
		return (Game)games.get(0);
	}
	public Player getPlayer(String name)
	{	for(int i=0;i<players.size();i++)
			if(((Player)players.get(i)).getPlayerName().equals(name))
				return (Player)players.get(i);
		return (Player)players.get(0);
	}
	public String seekStrings()
	{	String r="";
		for(int i=0;i<seeks.size();i++)
			r+=seeks.get(i).toString();
		return r;
	}
	public String playerStrings()
	{	String r="";
		for(int i=0;i<players.size();i++)
			r+=((Player)players.get(i)).getPlayerName()+"\n";
		return r;
	}
	public void chat(String s)
	{	for(int i=0;i<players.size();i++)
		{	((Player)players.get(i)).chat(s);
		}
	}
	public void playersUpdated()
	{	String p=playerStrings();
		for(int i=0;i<players.size();i++)
			((Player)players.get(i)).playersUpdated(p);
	}
	public void seeksUpdated()
	{	String s=seekStrings();
		for(int i=0;i<players.size();i++)
			((Player)players.get(i)).seeksUpdated(s);
	}
	public synchronized void debugInput(String p,String s)
	{	debug.addInput(p+":"+s);
	}
	public synchronized void debugOutput(String p,String s)
	{	debug.addOutput(p+":"+s);
	}
}